# SPDX-FileCopyrightText: Copyright (c) 2022-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
# SPDX-License-Identifier: MIT

"""Exposes the AIR API client module"""

from .air_api import *  # noqa: F403
from . import v2  # noqa: F401
